NO_ATHINA_API_KEY = """
Please sign up at https://athina.ai and set an Athina API key.

See https://docs.athina.ai/evals/quick_start for more info.
"""

NO_OPENAI_API_KEY = """
Please set an OpenAI API key.

See https://docs.athina.ai/evals/quick_start for more info.
"""
